/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* Program:                                                            */
/* --------                                                            */
/*                                                                     */
/* nbplotl.c                                                           */
/*                                                                     */
/*                                                                     */
/* Version:                                                            */
/* --------                                                            */
/*                                                                     */
/* First version: February 2000                                        */
/* Latest update: 2006-08-29                                           */
/*                                                                     */
/*                                                                     */
/* Author of the program:                                              */
/* ----------------------                                              */
/*                                                                     */
/* Stefan Spaennare, Lund, Sweden                                      */
/*                                                                     */
/* E-mail: stefan@spaennare.se                                         */
/*                                                                     */
/* Web: http://www.spaennare.se/index.html                             */
/* Web: http://www.spaennare.se/ssnbody.html                           */
/*                                                                     */
/* If you have problems with the program please let me know!           */
/*                                                                     */
/*                                                                     */
/* Documentation:                                                      */
/* --------------                                                      */
/*                                                                     */
/* For a detailed description of the ssnbody nbody package see the     */
/* files ssnbody.pdf and ssnbody.ps on the ssnbody web page.           */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "allegro.h"

double timetic=(double)(CLOCKS_PER_SEC);

char cfg_name[21]="nbplotl.cfg";

char image_name[21]="nbplotl.bmp";


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *infile,*cfgfile;

   BITMAP *bmp;
   PALETTE pal;

   int i,j,n,delay,lw,flag,cmod;
   int xsize,ysize,xi1,yi1,xi2,yi2;
   int col,c0,wintype,imsave,bgcol,bordercol;

   double xx1,xx2,yy1,yy2;
   double x,y,z,m,ftime;
   double sum,totsum;
   double xorig,yorig,xss,yss,xoo,yoo;
   double xp,yp;

   int cr[21];
   int cg[21];
   int cb[21];

   clock_t time1,timediff;

   double *x2;
   double *y2;

   if (argc != 10) {
printf("Usage: %s plot-infile(x y z m) x1r y1r x2r y2r delayi line-widthi xyz-flagi cmodi\n",argv[0]);
      printf("\n");
      printf("xyz-flag = 1 => x y\n");
      printf("xyz-flag = 2 => x z\n");
      printf("xyz-flag = 3 => y z\n");
      exit(0);
   } /* if */


   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File %s not found.\n",argv[1]);
      exit(0);
   } /* if */

   xx1=atof(argv[2]);
   yy1=atof(argv[3]);
   xx2=atof(argv[4]);
   yy2=atof(argv[5]);
   delay=atoi(argv[6]);
   lw=atoi(argv[7]);
   flag=atoi(argv[8]);
   cmod=atoi(argv[9]);


   if ((cfgfile = fopen(cfg_name,"r")) == NULL) {
      printf("File %s not found.\n",cfg_name);
      exit(0);
   } /* if */

   fscanf(cfgfile,"%d %d",&xsize,&ysize);
   fscanf(cfgfile,"%d",&wintype);
   fscanf(cfgfile,"%d",&imsave);

   fscanf(cfgfile,"%d",&bgcol);
   fscanf(cfgfile,"%d",&bordercol);


   if ((wintype < 0) || (wintype > 1)) {
      printf("wintype must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((imsave < 0) || (imsave > 1)) {
      printf("imsave must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((bordercol < 0) || (bordercol > 14)) {
      printf("bordercol must be between 0 or 14.\n");
      exit(0);
   } /* if */


   allegro_init();

   install_keyboard();

   set_color_depth(32);


   if (wintype == 0) {
      if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */

   if (wintype == 1) {
      if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */


   set_palette(desktop_palette);


   cr[0]=0;    cg[0]=0;    cb[0]=0;
   cr[1]=255;  cg[1]=255;  cb[1]=255;

   cr[2]=255;  cg[2]=0;    cb[2]=0;
   cr[3]=0;    cg[3]=255;  cb[3]=0;
   cr[4]=0;    cg[4]=0;    cb[4]=255;
   cr[5]=255;  cg[5]=0;    cb[5]=255;
   cr[6]=255;  cg[6]=255;  cb[6]=0;
   cr[7]=0;    cg[7]=255;  cb[7]=255;
   cr[8]=128;  cg[8]=0;    cb[8]=0;
   cr[9]=0;    cg[9]=128;  cb[9]=0;
   cr[10]=0;   cg[10]=0;   cb[10]=128;
   cr[11]=128; cg[11]=128; cb[11]=0;
   cr[12]=128; cg[12]=0;   cb[12]=128;
   cr[13]=0;   cg[13]=128; cb[13]=128;
   cr[14]=128; cg[14]=128; cb[14]=128;


   time1=clock();

   c0=bgcol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   rectfill(screen,0,0,xsize-1,ysize-1,col);

   c0=bordercol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   line(screen,0,0,xsize-1,0,col);
   line(screen,xsize-1,0,xsize-1,ysize-1,col);
   line(screen,xsize-1,ysize-1,0,ysize-1,col);
   line(screen,0,ysize-1,0,0,col);

   fscanf(infile,"%d",&n);

   if (cmod < 1) {
      cmod=1;
   } /* if */

   if (cmod > n) {
      cmod=n;
   } /* if */

   if (cmod > 12) {
      cmod=12;
   } /* if */

   if ((flag < 1) || (flag > 3)) {
      flag=1;
   } /* if */

   xoo=0.5*(xx1+xx2);
   yoo=0.5*(yy1+yy2);
    
   xorig=0.5*(double)(xsize);
   yorig=0.5*(double)(ysize);

   xss=(double)(ysize)/(xx2-xx1);
   yss=(double)(ysize)/(yy2-yy1);

   x2=(double *)calloc(n+1,sizeof(double));
   y2=(double *)calloc(n+1,sizeof(double));

   for (i=1; i<=n; i++) {
      fscanf(infile,"%lf %lf %lf %lf",&x,&y,&z,&m);
      if (flag == 1) {
         x2[i]=x;
         y2[i]=y;
      } /* if */
      if (flag == 2) {
         x2[i]=x;
         y2[i]=z;
      } /* if */
      if (flag == 3) {
         x2[i]=y;
         y2[i]=z;
      } /* if */
   } /* for i */

   totsum=0.0;

   i=1;

   while ((fscanf(infile,"%lf %lf %lf %lf",&x,&y,&z,&m) != EOF) &&
         (!keypressed())) {

      if (flag == 1) {
         xp=x;
         yp=y;
      } /* if */
      if (flag == 2) {
         xp=x;
         yp=z;
      } /* if */
      if (flag == 3) {
         xp=y;
         yp=z;
      } /* if */

      xi1=(int)(xss*(x2[i]-xoo)+xorig+0.5);
      yi1=(int)(yss*(y2[i]-yoo)+yorig+0.5);
      yi1=ysize-yi1;

      
      xi2=(int)(xss*(xp-xoo)+xorig+0.5);
      yi2=(int)(yss*(yp-yoo)+yorig+0.5);
      yi2=ysize-yi2;

      col=((i-1) % cmod)+2;

      line(screen,xi1,yi1,xi2,yi2,makecol(cr[col],cg[col],cb[col]));

      x2[i]=xp;
      y2[i]=yp;

      sum=0.0;

      for (j=1; j<=delay; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      totsum=totsum+sum;

      if (i == n) {
         i=0;
      } /* if */

      i++;


   } /* while */


   while (!keypressed()) {
   } /* while */


   c0=bordercol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   line(screen,0,0,xsize-1,0,col);
   line(screen,xsize-1,0,xsize-1,ysize-1,col);
   line(screen,xsize-1,ysize-1,0,ysize-1,col);
   line(screen,0,ysize-1,0,0,col);


   if (imsave == 1) {
      get_palette(pal);
      bmp=create_sub_bitmap(screen,0,0,xsize,ysize);
      save_bitmap(image_name,bmp,pal);
      destroy_bitmap(bmp);
   } /* if */


   allegro_exit();


   timediff=clock()-time1;

   ftime=(double)(timediff)/timetic;
   printf("\nTime:  %12.2f\n",ftime);
   printf("\nDummy:  %8.3e\n",totsum);

   fclose(infile);
   fclose(cfgfile);

   return(0);

} /* End */

END_OF_MAIN();


