/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* Program:                                                            */
/* --------                                                            */
/*                                                                     */
/* ring.c                                                              */
/*                                                                     */
/*                                                                     */
/* Version:                                                            */
/* --------                                                            */
/*                                                                     */
/* First version: February 2000                                        */
/* Latest update: 2003-02-18                                           */
/*                                                                     */
/*                                                                     */
/* Author of the program:                                              */
/* ----------------------                                              */
/*                                                                     */
/* Stefan Spaennare, Lund, Sweden                                      */
/*                                                                     */
/* E-mail: stefan@spaennare.se                                         */
/*                                                                     */
/* Web: http://www.spaennare.se/index.html                             */
/* Web: http://www.spaennare.se/ssnbody.html                           */
/*                                                                     */
/* If you have problems with the program please let me know!           */
/*                                                                     */
/*                                                                     */
/* Documentation:                                                      */
/* --------------                                                      */
/*                                                                     */
/* For a detailed description of the ssnbody nbody package see the     */
/* files ssnbody.pdf and ssnbody.ps on the ssnbody web page.           */
/*                                                                     */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/



#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double pi=3.14159265358979323;


void ring(r1,nr1,m1,x1,y1,vx1,vy1,gg,outfile)
double r1;
long nr1;
double m1,x1,y1,vx1,vy1,gg;
FILE *outfile;
{

  long i,j,n;

  double rr,v,alfa,x,y,z;
  double r2,dx,dy,dz,r,r3;
  double ax,ay,az,aaa,temp,mm,mmm;

  double *xx1,*yy1,*zz1;
  double *vxx1,*vyy1,*vzz1;
  double *mm0;


  n=nr1;

  printf("%6d\n",n);

  xx1=(double *)calloc(n+1,sizeof(double));
  yy1=(double *)calloc(n+1,sizeof(double));
  zz1=(double *)calloc(n+1,sizeof(double));
  vxx1=(double *)calloc(n+1,sizeof(double));
  vyy1=(double *)calloc(n+1,sizeof(double));
  vzz1=(double *)calloc(n+1,sizeof(double));
  mm0=(double *)calloc(n+1,sizeof(double));

  rr=r1;
  mmm=m1/(double)(n);

  alfa=0.0;


  for (i=1; i<=n; i++) {

     xx1[i]=rr*cos(alfa)+x1;
     yy1[i]=rr*sin(alfa)+y1;
     zz1[i]=0.0;

     mm0[i]=mmm;

     alfa=alfa+2.0*pi/(double)(n);

  } /* for i */

alfa=0.0;

for (j=1; j<=n; j++) {

  x=xx1[j];
  y=yy1[j];
  z=zz1[j];

  ax=0.0;
  ay=0.0;
  az=0.0;

  for (i=1; i<=n; i++) {

    if (i != j) {

       dx=x-xx1[i];
       dy=y-yy1[i];
       dz=z-zz1[i];
       mm=mm0[i];

       r2=dx*dx+dy*dy+dz*dz;
       r=sqrt(r2);
       r3=1.0/(r*r2);

       temp=gg*mm*r3;

       ax=ax+temp*dx;
       ay=ay+temp*dy;
       az=az+temp*dz;

    } /* if */

  } /* for i */


  aaa=sqrt(ax*ax+ay*ay+az*az);

  v=sqrt(aaa*rr);

  vxx1[j]=-v*sin(alfa)+vx1;
  vyy1[j]=v*cos(alfa)+vy1;
  vzz1[j]=0.0;

  alfa=alfa+2.0*pi/(double)(n);

} /* for j */


for (i=1; i<=n; i++) {

   fprintf(outfile,"%25.17le %25.17le %25.17le %25.17le %25.17le %25.17le %25.17le\n",
          xx1[i],yy1[i],zz1[i],vxx1[i],vyy1[i],vzz1[i],mm0[i]);

} /* for i */


free(xx1);
free(yy1);
free(zz1);
free(vxx1);
free(vyy1);
free(vzz1);
free(mm0);

} /* ring */


int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile,*outfile;

   long nr1;

   double r1,m1,x1,y1,vx1,vy1,gg;

   if (argc != 3) {
      printf("Usage: %s infile outfile\n",argv[0]);
      printf("\n");
      printf("Infile parameters:\n");
      printf("       gr\n");
      printf("       r1r nr1i m1r x1r y1r vx1r vy1r\n");
      printf("       r2r nr2i m2r x2r y2r vx2r vy2r\n");
      printf("                  ...\n");
      printf("       rnr nrni mnr xnr ynr vxnr vynr\n");
      exit(0);
   } /* if */

   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile = fopen(argv[2],"w");

   fscanf(infile,"%lf",&gg);

   while (fscanf(infile,"%lf %d %lf %lf %lf %lf %lf",
                 &r1,&nr1,&m1,&x1,&y1,&vx1,&vy1) != EOF) {

      ring(r1,nr1,m1,x1,y1,vx1,vy1,gg,outfile);

   } /* while */

   fclose(infile);
   fclose(outfile);

} /* End */
