/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* Program:                                                            */
/* --------                                                            */
/*                                                                     */
/* eq2eclp.c                                                           */
/*                                                                     */
/*                                                                     */
/* Version:                                                            */
/* --------                                                            */
/*                                                                     */
/* First version: 2003-01-28                                           */
/* Latest update: 2006-09-07                                           */
/*                                                                     */
/*                                                                     */
/* Author of the program:                                              */
/* ----------------------                                              */
/*                                                                     */
/* Stefan Spaennare, Lund, Sweden                                      */
/*                                                                     */
/* E-mail: stefan@spaennare.se                                         */
/*                                                                     */
/* Web: http://www.spaennare.se/index.html                             */
/* Web: http://www.spaennare.se/ssnbody.html                           */
/*                                                                     */
/* If you have problems with the program please let me know!           */
/*                                                                     */
/*                                                                     */
/* Documentation:                                                      */
/* --------------                                                      */
/*                                                                     */
/* For a detailed description of the ssnbody nbody package see the     */
/* files ssnbody.pdf and ssnbody.ps on the ssnbody web page.           */
/*                                                                     */
/*                                                                     */
/* References:                                                         */
/* ===========                                                         */
/*                                                                     */
/* The book "Practical Astronomy with Your Calculator", Third Edition, */
/* by Peter Duffett-Smith, 1988.                                       */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>


double pi=3.14159265358979323;


/* Converts radians to degrees. */

double deg(x)
double x;
{
   return(180.0*x/pi);
} /* deg */


/* Converts degrees to radians. */

double rad(x)
double x;
{
  return(pi*x/180.0);
} /* rad */


/* Calculates Julian day. */

double jd(year,month,day)
int year,month,day;
{
   double f,g,a,jdd;

   if (month>=3) {
      f=(double)(year);
      g=(double)(month);
   }
   else {
      f=(double)(year-1);
      g=(double)(month+12);
   } /* if */

   a=2.0-floor(f/100.0)+floor(f/400.0);

   jdd=floor(365.25*f)+floor(30.6001*(g+1.0))+
       (double)(day)+a+1720994.5;

   return(jdd);

} /* jd */


int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile,*outfile;

   int i,n,year,month,day;

   double eps,ce,se,t,jd0,eps2000;

   double v[8];
   double w[8];

   if (argc < 4) {
      printf("\n");
      printf("Usage 1: %s ecl-plot-infile eq-plot-outfile julian-dayr\n",argv[0]);
      printf("\n");
      printf("Usage 2: %s ecl-plot-infile eq-plot-outfile -date yeari monthi dayi\n",argv[0]);
      printf("\n");
      exit(0);
   } /* if */

   if (strcmp(argv[3],"-date") == 0) {

      year=atoi(argv[4]);
      month=atoi(argv[5]);
      day=atoi(argv[6]);

      jd0=jd(year,month,day);

   }
   else {

      jd0=atof(argv[3]);

   } /* if */

   /* Infile contains ecliptic position and mass for the Sun  */
   /* planets, the Moon (and optional some asteroids).         */
   
   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File %s not found.\n",argv[1]);
      exit(0);
   } /* if */

   /* Outfile contains equatorial coordinates for the bodies. */

   outfile=fopen(argv[2],"w");

   /* eps = obliquity of the ecliptic as function of time. */

   eps2000=23.0*3600.0+26.0*60.0+21.45;

   eps2000=eps2000/3600.0;

   t=(jd0-2451545.0)/36525.0;

   eps=23.0*3600.0+26.0*60.0+21.45-46.815*t-0.0006*t*t+0.00181*t*t*t;

   eps=eps/3600.0;

   printf("\n");
   printf("eps = %14.10lf at actual date.\n",eps);
   printf("eps = %14.10lf at 2000 January 1.5 (jd 2451545.0).\n",eps2000);
   printf("\n");

   eps=rad(eps);

   ce=cos(eps);
   se=sin(eps);

   /* Read and print the number of bodies. */

   fscanf(infile,"%d",&n);

   fprintf(outfile,"%8d\n",n);

   /* Read ecliptic data for the Sun, planets and the Moon    */
   /* (position and mass).                                    */

   while (fscanf(infile,"%lf %lf %lf %lf",
                 &v[1],&v[2],&v[3],&v[4]) != EOF) {


      /* Perform transformation from ecliptic to equatorial coordinates.*/

      w[1]=v[1];
      w[2]=ce*v[2]-se*v[3];
      w[3]=se*v[2]+ce*v[3];

      w[4]=v[4];

      /* Print equatorial coordinates (position and mass). */

      fprintf(outfile,"%13.6le %13.6le %13.6le %13.6le\n"
	     ,w[1],w[2],w[3],w[4]);

      i++;

   } /* while */

   fclose(infile);
   fclose(outfile);

} /* End */
